#ifndef DS_BASICIOS__HPP
#define DS_BASICIOS__HPP

#include <ds_streambuf.hpp>
#include <string>

namespace DS_NAMESPACE
{
  extern char *truestr;
  extern char *falsestr;
  extern wchar_t *truewstr;
  extern wchar_t *falsewstr;

  extern char *uppernumbers;
  extern char *lowernumbers;
  extern wchar_t *upperwnumbers;
  extern wchar_t *lowerwnumbers;

  template <TYPENAME charT>
  class basic_ios_traits
  {
  public:
    charT *truestring, *falsestring;
    charT *uppernumber, *lowernumber;
    basic_ios_traits() :
      truestring(0), falsestring(0),
      uppernumber(0), lowernumber(0)
    { }
  };

  template <>
  class basic_ios_traits<char>
  {
  public:
    char *truestring, *falsestring;
    char *uppernumber, *lowernumber;
    basic_ios_traits() :
      truestring(truestr), falsestring(falsestr),
      uppernumber(uppernumbers), lowernumber(lowernumbers)
    { }
  };

  template <>
  class basic_ios_traits<wchar_t>
  {
  public:
    wchar_t *truestring, *falsestring;
    wchar_t *uppernumber, *lowernumber;
    basic_ios_traits() :
      truestring(truewstr), falsestring(falsewstr),
      uppernumber(upperwnumbers), lowernumber(lowerwnumbers)
    { }
  };

  template <TYPENAME charT>
  class basic_ios : public ios_base
  {
  public:
    typedef basic_ios<charT>* tie_t;

  protected:
    enum {
      BIOSERR = -2000000,
    };

  private:
    /// if this flag is set the sb_ pointer is not touched in destructor
    bool dontcleanupsb;

    tie_t tie_;
    void flushtie()
    {
      if(tie_ && sb_)
	tie_->sync();
    }

    void sync()
    {
      if(sb_)
	sb_->flush();
    }

  protected:
    streambuf *sb_;
    int sbput(const int i)
    {
      flushtie();

      if(!sb_)
	return BIOSERR;

      const int c=sb_->put(i);
      if(c==streambuf::SBEOF)
	return BIOSERR;
      return 0;
    }

    int sbget()
    {
      flushtie();

      if(!sb_)
	return BIOSERR;

      const int c=sb_->get();
      if(c==streambuf::SBEOF)
	return BIOSERR;
      return c;
    }

  public:

    basic_ios<charT>(streambuf *s=0, tie_t t=0) :
      dontcleanupsb(false),
      tie_(t),
      sb_(0)
    {
      init(s);
    }

    ~basic_ios<charT>()
    {
      ds_debughook("~basic_ios()");
      if(sb_ && !dontcleanupsb && sb_->delref()<0)
	{
	  ds_debughook("~basic_ios(): delete");
	  delete sb_;
	}
      ds_debughook("~basic_ios(): finish");
    }

    streambuf* rdbuf() const { return sb_; }
    streambuf* rdbuf(streambuf *newsb)
    {
      streambuf *oldsb=sb_;
      if(oldsb)
	oldsb->delref();
      if(newsb)
	newsb->addref();
      sb_=newsb;
      if(sb_)
	setstate(goodbit);
      else
	setstate(badbit);
      return oldsb;
    }

    tie_t tie() const { return tie_; }
    tie_t tie(tie_t n)
    {
      tie_t o=tie_;
      tie_=n;
      return o;
    }

    basic_ios<charT>& copyfmt(const basic_ios<charT>& i)
    { 
      // buggy
      fill_=i.fill_;
      fmtflags_=i.fmtflags_;
      rdbuf(i.rdbuf());
      return *this;
    }

    /**
       @param dcsb a private flag only used when basic_ios is used as cin, cout etc.
    */
    void init(streambuf *newsb, const bool dcsb=false)
    {
      dontcleanupsb=dcsb;
      rdbuf(newsb);
      fmtflags_=skipws|dec;
      width_=0;
      precision_=6;
      fill_=' ';
    }

  private:
    basic_ios<charT> (const basic_ios<charT>&); // we don't want copy construction
    basic_ios<charT>& operator= (const basic_ios<charT>&); // we don't want assign operations
  };

  typedef basic_ios<char> ios;

}

#endif
