#include <stdio.h>

#include <ds_filebuf.hpp>

namespace DS_NAMESPACE
{

  basic_filebufhelper::~basic_filebufhelper()
  { 
    close();
  }

  bool basic_filebufhelper::open(const char *fn, int flags)
  {
    close();
    closefile=true;

    if(!fn)
      return false;

    // either in or out must be set
    if(!(flags&(ios_base::in|ios_base::out)))
      return false;

    char mode[4]; int modeI=0;

    if((flags&ios_base::in) && (flags&ios_base::out))
      {
	if(flags&ios_base::trunc)
	  {
	    mode[modeI++]='w';
	    mode[modeI++]='+';
	  }
	else
	  {
	    mode[modeI++]='r';
	    mode[modeI++]='+';
	  }
      }
    else if(flags&ios_base::in)
      {
	mode[modeI++]='r';
      }
    else
      {
	if(flags&ios_base::app)
	  mode[modeI++]='a';
	else
	  mode[modeI++]='w';
      }

    if(flags&ios_base::binary)
      mode[modeI++]='b';
    else
      mode[modeI++]='t';

    mode[modeI++]=0;

    f=fopen(fn, mode);
    if(f==0)
      return false;
    if(flags&ios_base::ate)
      seek(0, ios_base::end);

    return true;
  }

  bool basic_filebufhelper::open(FILE *file)
  {
    close();
    f=file;
    closefile=false;
    return f?true:false;
  }

  bool basic_filebufhelper::close()
  {
    if(f && closefile)
      {
	fclose(f);
      }

    f=0;
    return true;
  }

  bool basic_filebufhelper::is_open()
  {
    return f?true:false;
  }

  streampos basic_filebufhelper::pos()
  {
    if(!f)
      return 0;
    return ftell(f);
  }

  int basic_filebufhelper::seek(const streamoff offs, const ios_base::seek_dir flag)
  {
    if(!f)
      return SBEOF;
    int i=0;
    switch(flag)
      {
      case ios_base::beg: i=SEEK_SET; break;
      case ios_base::cur: i=SEEK_CUR; break;
      case ios_base::end: i=SEEK_END; break;
      default: return false;
      }
    if(fseek(f, offs, i)==0)
      return pos();
    return SBEOF;
  }

  int basic_filebufhelper::flush()
  {
    if(!f)
      return SBEOF;
    return (fflush(f)==0)?0:SBEOF;
  }

#define FILEBUFPRIVATE

#define CHART char
#define CLASS basic_filebuf

#include "ds_filebufprivate.cpp"

#undef CHART
#undef CLASS

#define CHART wchar_t
#define CLASS basic_wfilebuf

#include "ds_filebufprivate.cpp"

}
