#include <ds_basicistream.hpp>
#include <ds_basicostream.hpp>
#include <ds_filebuf.hpp>

#include <stdio.h>

namespace DS_NAMESPACE
{
  istream cin;
  ostream cout;
  ostream cerr;
  ostream clog;
  wistream wcin;
  wostream wcout;
  wostream wcerr;
  wostream wclog;

  ios_base::Init::Init()
  {
#if !defined(_WIN32_WCE)
    Realinit();
#endif
  }

  void ios_base::Init::Realinit()
  {
    static int count=0;
    if(count==0)
      {
	basic_filebuf* insb=new basic_filebuf;
	insb->delref();		// let the stream objects delete insb
	insb->open(stdin);
	cin.rdbuf(insb);

	basic_filebuf* outsb=new basic_filebuf;
	outsb->delref();	// let the stream objects delete outsb
	outsb->open(stdout);
	cout.rdbuf(outsb);

	basic_filebuf* errsb=new basic_filebuf;
	errsb->delref();	// let the stream objects delete errsb
	errsb->open(stderr);
	cerr.rdbuf(errsb);
	clog.rdbuf(errsb);

	cin.tie(&cout);

	basic_wfilebuf* winsb=new basic_wfilebuf;
	winsb->delref();
	winsb->open(stdin);
	wcin.rdbuf(winsb);

	basic_wfilebuf* woutsb=new basic_wfilebuf;
	woutsb->delref();
	woutsb->open(stdout);
	wcout.rdbuf(woutsb);

	basic_wfilebuf* werrsb=new basic_wfilebuf;
	werrsb->delref();
	werrsb->open(stderr);
	wcerr.rdbuf(werrsb);
	wclog.rdbuf(werrsb);

	wcin.tie(&wcout);

	++count;
      }
  }

  void ios_base::sync_with_stdio(bool sync)
  {
    // todo: http://www.cplusplus.com/ref/iostream/ios_base/sync_with_stdio.html
  }

  char *truestr="true";
  char *falsestr="false";
  wchar_t *truewstr=L"true";
  wchar_t *falsewstr=L"false";

  char *uppernumbers=     "0123456789ABCDEFX";
  char *lowernumbers=     "0123456789abcdefx";
  wchar_t *upperwnumbers=L"0123456789ABCDEFX";
  wchar_t *lowerwnumbers=L"0123456789abcdefx";

  void ds_debughook(const char* str, const int i)
  {
#ifdef DEBUG
    fprintf(stderr, "ds_debughook():");
    fprintf(stderr, str, i);
    fprintf(stderr, "\n");
#endif
  }

  void unicode2latin(const std::wstring& u, std::string& l)
  {
    for(std::wstring::const_iterator i=u.begin(); i!=u.end(); ++i)
      {
	wchar_t c=*i;
	if(c<256)
	  {
	    unsigned char ch=static_cast<unsigned char>(c);
	    l+=*reinterpret_cast<char*>(&ch);
	  }
	else
	  l+='.';
      }
  }

  void latin2unicode(const std::string& l, std::wstring& u)
  {
    for(std::string::const_iterator i=l.begin(); i!=l.end(); ++i)
      {
	wchar_t c=*i;
	u+=c;
      }
  }

}
