#ifndef DS_STREAMITERATOR__HPP
#define DS_STREAMITERATOR__HPP

#include <ds_basicostream.hpp>
#include <ds_basicistream.hpp>

namespace DS_NAMESPACE
{
  template <TYPENAME T, TYPENAME charT = char>
  class istream_iterator
  {
  protected:
    basic_istream<charT> *is;
    bool ok;
    T value;

    void read()
    {
      if(is)
	{
	  (*is) >> value;
	  ok=is->good();
	}
    }

  public:
    istream_iterator<T,charT>()
      : is(0), ok(false)
    { }

    istream_iterator<T,charT>(basic_istream<charT>& s)
      : is(&s), ok(s.good())
    { }

    bool operator== (const istream_iterator<T,charT>& s) const
    {
      if(is && s.is)
	return is==s.is && ok==s.ok;
      return ok==s.ok;
    }

    bool operator!= (const istream_iterator<T,charT>& s) const
    {
      return !(*this==s);
    }
    const T& operator* () const { return value; }
    const T* operator->() const { return &value; }

    istream_iterator<T,charT>& operator++ ()
    {
      read();
      return *this;
    }

    istream_iterator<T,charT>& operator++ (int)
    {
      istream_iterator<T,charT> tmp=*this;
      read();
      return tmp;
    }
  };

  template <TYPENAME T, TYPENAME charT = char>
  class ostream_iterator
  {
  protected:
    basic_ostream<charT> *os;
    charT delim;
  public:
    ostream_iterator<T,charT>()
      : os(0)
    { }

    ostream_iterator<T,charT>(basic_ostream<charT>& s, charT d=' ')
      : os(&s), delim(d)
    { }

    ostream_iterator<T,charT>& operator= (const T& t)
    {
      if(os)
	{
	  (*os) << t;
	  (*os) << delim;
	}
      return *this;
    }

    ostream_iterator<T,charT>& operator++ ()    { return *this; }
    ostream_iterator<T,charT>& operator++ (int) { return *this; }
    ostream_iterator<T,charT>& operator*  ()    { return *this; }
  };

}

#endif
